#!/usr/bin/env python

#Open Projector (open-projector-gtk)
#Generates a flexible slideshow presentation using Glade and GTK
#
#Copyright (C) 2008, Dylan McCall
#
#This program is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program. If not, see <http://www.gnu.org/licenses/>

import os
import sys
import random

import gobject

import pygtk
pygtk.require('2.0')

import gtk
import gtk.glade

#FIXME: File paths are all joined via a stupid method (simply appending "/") due to laziness and forgetfulness.

class ConfigObj():
#Open-ended configuration system for disconnected key files...
	def __init__(self,configdir, defaults=None):
		self.configdir = configdir
		self.defaults = defaults
		self.configdict = dict()
	
	def GetValue(self,key):
		if key in self.configdict:
			#Recall value from memory if it has been placed there before
			return self.configdict[key]
		elif os.path.exists(self.configdir + '/' + key):
			#Read from the raw config file the first time
			keyfile = open(self.configdir + '/' + key, 'r')
			value = keyfile.read()
			self.SetValue(key,value) #Save value into memory for next time
			return value
		elif self.defaults:
			#Read from default configuration
			return self.defaults.GetValue(key)
		else:
			#Could not find key anywhere!
			return 0
	
	def SetValue(self,key,value):
		self.configdict[key] = value
	

class Slide():
	def __init__(self,slide_path):
		self.slide_path = slide_path
		
		self.LoadGUI()
		
		self.projector_widget = self.wTree.get_widget('projector')
		self.projector = None
		if self.projector_widget:
			self.projector = Projector(self.projector_widget,self.slide_path)
	
	def LoadGUI(self):
		self.gladefile = os.path.join(self.slide_path,'slide.glade')
		self.wTree = gtk.glade.XML(self.gladefile,'slide')
		
		#self.wTree.signal_autoconnect(SOMETHING)
		
		self.widget = self.wTree.get_widget('slide')

class Projector():
	config_default = ConfigObj(None)
	config_default.SetValue('slide_end','special:end')
	config_default.SetValue('slide_start','special:start')
	config_default.SetValue('slides_order','alphabetical')
	config_default.SetValue('slides_duration_seconds','-1')
	
	def __init__(self,host_widget,path):
		self.path = path
		self.host_widget = host_widget
		
		self.LoadConfiguration()
		
		self.LoadSlides()
		self.StartProjector()
		
	def LoadConfiguration(self):
		self.config = ConfigObj(os.path.join(self.path,'_conf'),Projector.config_default)
	
	def LoadSlides(self):
		self.current_slide = None
		self.slides_paths = list()
		
		dirlist = os.listdir(self.path)
		
		if self.config.GetValue('slides_order') == 'alphabetical':
			dirlist.sort()
			
		elif self.config.GetValue('slides_order') == 'shuffle':
			random.shuffle(dirlist)
		
		for fname in dirlist:
			if os.path.isdir(os.path.join(self.path,fname)) and not fname.startswith('_') and not fname.startswith('special:'):
				self.slides_paths.append(os.path.join(self.path,fname)) #Place a full path to here in the actual list of slide paths, if this is a proper slide
		
		if self.config.GetValue('slide_start') in dirlist:
			slidepath = os.path.join(self.path,self.config.GetValue('slide_start'))
			if os.path.isdir(slidepath):
				self.slides_paths.insert(0,slidepath)
		
		if self.config.GetValue('slide_end') in dirlist:
			slidepath = os.path.join(self.path,self.config.GetValue('slide_end'))
			if os.path.isdir(slidepath):
				self.slides_paths.append(slidepath)
	
	def StartProjector(self):
		self.NextSlide() #Open the first slide to start us off...
		
		timerlength = int(self.config.GetValue('slides_duration_seconds'))*1000 #Timer must be in milliseconds
		projector_timeout = gobject.timeout_add(timerlength, self.NextSlide)
	
	def NextSlide(self):
		if self.slides_paths:
			slide_path = self.slides_paths.pop(0)
			self.ChangeSlide(slide_path)
			return True
		else:
			return False
	
	def ChangeSlide(self,slide_path):
		new_slide = Slide(slide_path)
		if self.current_slide:
			self.current_slide.widget.destroy()
		self.current_slide = new_slide
		self.host_widget.add(self.current_slide.widget)

def usage():
	print sys.argv[0] + ' slide_path'

if __name__ == '__main__':
	if len(sys.argv) > 1:
		main_window = gtk.Window()
		main_window.set_size_request(640,480)
		main_window.show_all()
		
		main_window.connect("destroy", gtk.main_quit)
		
		main_projector = Projector(main_window,sys.argv[1]) #Create main projector in slide_path, embedded in the hard-coded main window
		
		gtk.main()
	else:
		usage()
	
	sys.exit()